package com.io.yutian.colorblindwar;

import com.io.yutian.colorblindwar.game.DirectionPoint;
import com.io.yutian.colorblindwar.game.Game;
import com.io.yutian.colorblindwar.listener.PlayerListener;
import com.io.yutian.colorblindwar.live.GiftQueue;
import com.io.yutian.colorblindwar.manager.GiftEffectManager;
import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;

import java.util.Map;

public final class ColorblindWar extends JavaPlugin {

    private static ColorblindWar instance;
    private static Game game;

    @Override
    public void onEnable() {
        instance = this;

        GiftEffectManager.registerAll();

        game = new Game();
        Bukkit.getPluginManager().registerEvents(new PlayerListener(), this);
    }

    @Override
    public void onDisable() {
        if (game.isStarted()) {
            game.stop(false);
        }
        GiftEffectManager.stopQueue();
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        if (args.length == 0) {
            sender.sendMessage("/cw create - 创建游戏");
            sender.sendMessage("/cw start - 开始游戏");
            sender.sendMessage("/cw stop - 结束游戏");
            sender.sendMessage("/gw reload - 重载");
            return true;
        } else {
            String subCommand = args[0];
            if (subCommand.equalsIgnoreCase("create")) {
                if (!(sender instanceof Player)) {
                    sender.sendMessage("非玩家无法创建游戏");
                    return true;
                }
                Player player = (Player) sender;
                game.setWorld(player.getWorld());
                game.setSpawnPoint(DirectionPoint.of(player.getLocation()));
                game.reload();
                player.sendMessage("成功创建游戏");
            } else if (subCommand.equalsIgnoreCase("start")) {
                if (!(sender instanceof Player)) {
                    sender.sendMessage("非玩家无法开始游戏");
                    return true;
                }
                if (game.isStarted()) {
                    sender.sendMessage("游戏已开始");
                    return true;
                }
                Player player = (Player) sender;
                game.start(player);
            } else if (subCommand.equalsIgnoreCase("stop")) {
                if (!game.isStarted()) {
                    sender.sendMessage("游戏未开始");
                    return true;
                }
                Player player = (Player) sender;
                game.stop(false);
            } else if (subCommand.equalsIgnoreCase("reload")) {
                game.reload();
                sender.sendMessage("重载成功");
            } else if (subCommand.equalsIgnoreCase("test")) {
                if (!(sender instanceof Player)) {
                    return true;
                }
                String s1 = args[1];
                Player player = (Player) sender;
                if (s1.equalsIgnoreCase("add")) {
                    GiftEffectManager.addGiftEffect(player.getName(), "致盲");
                    player.sendMessage("add effect success");
                } else if (s1.equalsIgnoreCase("list")) {
                    player.sendMessage("effect list:");
                    for (Map.Entry<String, GiftQueue> entry : GiftEffectManager.getGiftQueues().entrySet()) {
                        String type = entry.getKey();
                        GiftQueue giftQueue = entry.getValue();
                        player.sendMessage(type+":");
                        player.sendMessage("  "+giftQueue.getEffectQueue().toString());
                    }
                }
            }
        }
        return true;
    }

    public static ColorblindWar inst() {
        return instance;
    }

    public static Game getGame() {
        return game;
    }

}
