package com.io.yutian.colorblindwar.game;

import com.io.yutian.colorblindwar.ColorblindWar;
import com.io.yutian.colorblindwar.util.*;
import com.sk89q.worldedit.EditSession;
import com.sk89q.worldedit.WorldEdit;
import com.sk89q.worldedit.bukkit.BukkitAdapter;
import com.sk89q.worldedit.function.mask.InverseSingleBlockTypeMask;
import com.sk89q.worldedit.function.mask.Mask;
import com.sk89q.worldedit.function.operation.Operations;
import com.sk89q.worldedit.math.BlockVector3;
import com.sk89q.worldedit.regions.CuboidRegion;
import com.sk89q.worldedit.world.block.BlockTypes;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.TextColor;
import org.bukkit.*;
import org.bukkit.boss.BarColor;
import org.bukkit.boss.BarStyle;
import org.bukkit.boss.BossBar;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Firework;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.FireworkMeta;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.scheduler.BukkitTask;

import java.util.*;

public class Game {

    private static final List<Material> MATERIAL_LIST = Arrays.asList(Material.WHITE_WOOL, Material.BLACK_WOOL, Material.BROWN_WOOL, Material.BLUE_WOOL, Material.GREEN_WOOL, Material.RED_WOOL, Material.YELLOW_WOOL, Material.PURPLE_WOOL, Material.ORANGE_WOOL, Material.PINK_WOOL, Material.LIGHT_BLUE_WOOL, Material.LIME_WOOL);

    private World world;
    private DirectionPoint spawnPoint;

    private Player player;

    private BossBar bossBar1;
    private BossBar bossBar2;
    private BossBar bossBar3;
    private BossBar bossBar4;

    private List<BukkitTask> tasks = new ArrayList<>();
    private List<Entity> entities = new ArrayList<>();

    private int challengeAmount = 0;
    private int vanishCountdown = 0;
    private int vanishCooldown = 0;
    private Material vanishMaterial = Material.AIR;

    private boolean started = false;

    private GameConfig gameConfig;
    private BuildSpawner buildSpawner;

    public Game() {
        reload();
        this.bossBar1 = Bukkit.createBossBar("今日挑战进度", BarColor.WHITE, BarStyle.SEGMENTED_10);
        this.bossBar2 = Bukkit.createBossBar("爬塔进度", BarColor.WHITE, BarStyle.SEGMENTED_20);
        this.bossBar3 = Bukkit.createBossBar("方块消失倒计时", BarColor.WHITE, BarStyle.SOLID);
        this.bossBar4 = Bukkit.createBossBar("方块出现倒计时", BarColor.WHITE, BarStyle.SOLID);
    }
    
    public void reload() {
        ColorblindWar.inst().saveDefaultConfig();
        ColorblindWar.inst().reloadConfig();
        FileConfiguration config = ColorblindWar.inst().getConfig();
        if (config.contains("world")) {
            this.world = Bukkit.getWorld(config.getString("world"));
        }
        if (config.contains("spawnPoint") && config.isConfigurationSection("spawnPoint")) {
            this.spawnPoint = DirectionPoint.deserialize(config.getConfigurationSection("spawnPoint"));
        }
        if (world == null || spawnPoint == null) {
            return;
        }
        BuildSpawner.Direction initialDirection = BuildSpawner.Direction.NORTH;
        int chunkWight = 2;
        int chunkLength = 2;
        int ladderLength = 3;
        int ladderAmount = 5;
        int platformWidth = 3;
        int platformLength = 7;
        int levelAmount = 10;
        BuildSpawner.SpawnMode spawnMode = BuildSpawner.SpawnMode.MODE_2;
        int completeAmount = config.getInt("completeAmount", 10);
        int countdownTime = config.getInt("countdownTime", 15);
        this.gameConfig = new GameConfig(initialDirection, chunkWight, chunkLength, ladderLength, ladderAmount, platformWidth, platformLength, levelAmount, spawnMode, MATERIAL_LIST, completeAmount, countdownTime);
        this.buildSpawner = gameConfig.getSpawner(world, spawnPoint);
        this.vanishCountdown = gameConfig.getCountdownTime() * 20;
    }

    public void start(Player player) {
        if (started) {
            return;
        }
        if (world == null || spawnPoint == null) {
            return;
        }
        this.started = true;
        this.player = player;
        this.challengeAmount = 0;
        this.vanishCountdown = gameConfig.getCountdownTime() * 20;
        initWorld();
        updateBossBar();
        this.bossBar1.addPlayer(player);
        this.bossBar2.addPlayer(player);
        this.bossBar3.addPlayer(player);
        this.bossBar4.addPlayer(player);
        spawnBuild();
        changeVanishMaterial();
        player.teleport(spawnPoint.toLocation(world));
        this.tasks.add(new BukkitRunnable() {
            @Override
            public void run() {
                updateBossBar();
            }
        }.runTaskTimer(ColorblindWar.inst(), 0L, 10L));
        this.tasks.add(new BukkitRunnable() {
            @Override
            public void run() {
                updateVanishCountdown();
            }
        }.runTaskTimer(ColorblindWar.inst(), 0L, 1L));
    }

    private void initWorld() {
        this.world.setGameRule(GameRule.DO_DAYLIGHT_CYCLE, false);
        this.world.setGameRule(GameRule.DO_WEATHER_CYCLE, false);
        this.world.setGameRule(GameRule.DO_MOB_SPAWNING, false);
        this.world.setGameRule(GameRule.DO_MOB_LOOT, false);
        this.world.setGameRule(GameRule.MOB_GRIEFING, false);
        this.world.setTime(6000);
    }

    public void stop(boolean win) {
        if (!started) {
            return;
        }
        this.started = false;
        for (Entity entity : entities) {
            entity.remove();
        }

        spawnBuild();
        this.bossBar1.removeAll();
        this.bossBar2.removeAll();
        this.bossBar3.removeAll();
        this.bossBar4.removeAll();
        this.player.getInventory().clear();
        this.player.teleport(spawnPoint.toLocation(world));
        this.entities.clear();
        if (win) {
            new BukkitRunnable() {
                private int i = 0;
                @Override
                public void run() {
                    if (i >= 20) {
                        cancel();
                        return;
                    }
                    player.getWorld().spawn(player.getLocation(), Firework.class, (firework -> {
                        FireworkMeta fireworkMeta = firework.getFireworkMeta();
                        fireworkMeta.addEffect(FireworkEffect.builder().withColor(Color.TEAL, Color.RED, Color.GREEN, Color.YELLOW, Color.LIME).trail(true).build());
                        Random random = new Random();
                        for (int i = 0; i < RandomUtil.getRandomInt(1, 2); i++) {
                            FireworkEffect effect = FireworkEffect.builder().flicker(random.nextBoolean()).withColor(Color.fromRGB(RandomUtil.getRandomInt(100, 255), RandomUtil.getRandomInt(100, 255), RandomUtil.getRandomInt(100, 255))).withFade(Color.fromRGB(RandomUtil.getRandomInt(100, 255), RandomUtil.getRandomInt(100, 255), RandomUtil.getRandomInt(100, 255))).with(FireworkEffect.Type.values()[random.nextInt(FireworkEffect.Type.values().length)]).trail(random.nextBoolean()).build();
                            fireworkMeta.addEffect(effect);
                        }
                        fireworkMeta.setPower(random.nextInt(2));
                        firework.setFireworkMeta(fireworkMeta);
                    }));
                    i++;
                }
            }.runTaskTimer(ColorblindWar.inst(), 0L, 5L);
            player.sendMessage("§c[系统]§ a今日挑战已完成! 恭喜你!");
            start(player);
        } else {
            player.sendMessage("§c[系统] §a游戏已被强制结束!");
        }
        tasks.forEach(task -> {
            if (!task.isCancelled()) {
                task.cancel();
            }
        });
    }

    private void updateBossBar() {
        int completeAmount = gameConfig.getCompleteAmount();
        double d = (double) challengeAmount / (double) completeAmount;
        bossBar1.setTitle("§6今日挑战进度: §f"+challengeAmount+"/"+completeAmount);
        if (d < 0) {
            d = 0;
        }
        bossBar1.setProgress(d);
        bossBar1.setColor(BossBarUtil.getBarColor(d));
        int maxHeight = spawnPoint.getBlockY() + buildSpawner.getHeight();
        int nowHeight = player.getLocation().getBlockY();
        int c1 = nowHeight - spawnPoint.getBlockY();
        c1 = Math.max(c1, 0);
        double d1 = (double) c1 / (double) buildSpawner.getHeight();
        d1 = Math.min(d1, 1.0);
        double d2 = MathUtil.round(d1 * 100, 2);
        bossBar2.setTitle("§6爬塔进度: §f"+c1+"/"+buildSpawner.getHeight()+" §b("+d2+"%)");
        bossBar2.setProgress(d1);
        bossBar2.setColor(BossBarUtil.getBarColor(d1));
        sendActionBar();
    }

    private void updateVanishCountdown() {
        if (vanishCooldown > 0) {
            vanishCooldown--;
            if (bossBar3.isVisible()) {
                bossBar3.setVisible(false);
            }
            if (!bossBar4.isVisible()) {
                bossBar4.setVisible(true);
            }
            bossBar4.setTitle("§6方块出现倒计时: §f"+ MathUtil.round(vanishCooldown / (double) 20, 1) +"s");
            double p = vanishCooldown / (double) 60;
            bossBar4.setProgress(p);
            bossBar4.setColor(BossBarUtil.getBarColor(p));
            if (vanishCooldown == 0) {
                bossBar4.setVisible(false);
                startVanishCountdown();
                return;
            }
            return;
        } else {
            if (!bossBar3.isVisible()) {
                bossBar3.setVisible(true);
            }
            if (bossBar4.isVisible()) {
                bossBar4.setVisible(false);
            }
        }
        if (vanishCountdown > 0) {
            vanishCountdown--;
            if (vanishCountdown <= 100 && vanishCountdown % 20 == 0) {
                player.playSound(player.getLocation(), Sound.BLOCK_NOTE_BLOCK_GUITAR, 1.0F, 1.0F);
            }
            bossBar3.setTitle("§6方块消失倒计时: §f"+ MathUtil.round(vanishCountdown / (double) 20, 1) +"s");
            double p = vanishCountdown / (double) (gameConfig.getCountdownTime() * 20);
            bossBar3.setProgress(p);
            bossBar3.setColor(BossBarUtil.getBarColor(p));
        } else {
            vanishBlock();
            for (int i = 0; i < 9; i++) {
                player.getInventory().setItem(i, null);
            }
            vanishCooldown = 60;
            bossBar3.setVisible(false);
        }
    }

    private void startVanishCountdown() {
        vanishCountdown = gameConfig.getCountdownTime() * 20;
        spawnBuild();
        changeVanishMaterial();
    }

    private void changeVanishMaterial() {
        vanishMaterial = buildSpawner.getRandomMaterial();
        ItemStack itemStack = new ItemStack(vanishMaterial);
        ItemMeta itemMeta = itemStack.getItemMeta();
        itemMeta.setDisplayName("");
        itemStack.setItemMeta(itemMeta);
        for (int i = 0; i < 9; i++) {
            player.getInventory().setItem(i, itemStack);
        }
    }

    private void sendActionBar() {
        Component message = Component.text("§l"+ ColorUtil.getColorName(vanishMaterial))
                .color(TextColor.fromHexString(ColorUtil.getColorHex(vanishMaterial)));
        player.sendActionBar(message);
    }

    public void spawnBuild() {
        buildSpawner.spawn();
    }

    public void vanishBlock() {
        Material material = vanishMaterial;
        Region region = buildSpawner.getEffectiveRegion(world);
        Point minPoint = region.getMin();
        Point maxPoint = region.getMax();
        BlockVector3 newVector1 = BlockVector3.at(minPoint.getX(), minPoint.getY(), minPoint.getZ());
        BlockVector3 newVector2 = BlockVector3.at(maxPoint.getX(), maxPoint.getY(),  maxPoint.getZ());
        CuboidRegion cuboidRegion = new CuboidRegion(BukkitAdapter.adapt(region.getWorld()),newVector1,newVector2);
        try (EditSession editSession = WorldEdit.getInstance().newEditSession(BukkitAdapter.adapt(region.getWorld()))) {
            Mask mask = new InverseSingleBlockTypeMask(editSession, BukkitAdapter.asBlockType(material));
            editSession.replaceBlocks(cuboidRegion, mask, BlockTypes.AIR.getDefaultState());
            Operations.complete(editSession.commit());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void clearBlock() {
        Region region = buildSpawner.getEffectiveRegion(world);
        Point minPoint = region.getMin();
        Point maxPoint = region.getMax();
        BlockVector3 newVector1 = BlockVector3.at(minPoint.getX(), minPoint.getY(), minPoint.getZ());
        BlockVector3 newVector2 = BlockVector3.at(maxPoint.getX(), maxPoint.getY(),  maxPoint.getZ());
        CuboidRegion cuboidRegion = new CuboidRegion(BukkitAdapter.adapt(region.getWorld()),newVector1,newVector2);
        try (EditSession editSession = WorldEdit.getInstance().newEditSession(BukkitAdapter.adapt(region.getWorld()))) {
            editSession.setBlocks((com.sk89q.worldedit.regions.Region) cuboidRegion, BlockTypes.AIR.getDefaultState());
            Operations.complete(editSession.commit());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void setWorld(World world) {
        this.world = world;
        FileConfiguration config = ColorblindWar.inst().getConfig();
        config.set("world", world.getName());
        ColorblindWar.inst().saveConfig();
    }

    public void setSpawnPoint(DirectionPoint spawnPoint) {
        this.spawnPoint = spawnPoint;
        FileConfiguration config = ColorblindWar.inst().getConfig();
        ConfigurationSection section1 = config.getConfigurationSection("spawnPoint");
        if(section1 == null){
            config.createSection("spawnPoint");
        }
        section1.set("x", spawnPoint.getX());
        section1.set("y", spawnPoint.getY());
        section1.set("z", spawnPoint.getZ());
        section1.set("yaw", spawnPoint.getYaw());
        section1.set("pitch", spawnPoint.getPitch());
        ColorblindWar.inst().saveConfig();
    }

    public boolean isStarted() {
        return started;
    }

    public Player getPlayer() {
        return player;
    }

    public World getWorld() {
        return world;
    }

    public Point getSpawnPoint() {
        return spawnPoint;
    }

    public GameConfig getGameConfig() {
        return gameConfig;
    }

    public BuildSpawner getBuildSpawner() {
        return buildSpawner;
    }

}
