package com.io.yutian.colorblindwar.manager;

import com.io.yutian.colorblindwar.game.Game;
import com.io.yutian.colorblindwar.live.GiftEffect;
import com.io.yutian.colorblindwar.live.GiftQueue;
import com.io.yutian.colorblindwar.util.RandomUtil;
import org.bukkit.entity.Player;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.bukkit.scheduler.BukkitTask;

import java.util.*;
import java.util.function.Function;

public class GiftEffectManager {

    private static Map<String, Function<String, GiftEffect>> giftEffectMap = new HashMap<>();

    private static Map<String, GiftQueue> giftQueues = new HashMap<>();

    public static void addGiftEffect(String audience, String effectId) {
        GiftEffect giftEffect = giftEffectMap.get(effectId).apply(audience);
        GiftQueue giftQueue = giftQueues.getOrDefault(effectId, new GiftQueue(giftEffect.getQueueTime()));
        giftQueue.addEffect(giftEffect);
        giftQueues.put(effectId, giftQueue);
    }

    public static void stopQueue() {
        giftQueues.values().forEach(giftQueue -> {
            BukkitTask task = giftQueue.getTask();
            if (task != null && !task.isCancelled()) {
                task.cancel();
            }
        });
    }

    public static GiftQueue getGiftQueue(String effectId) {
        return giftQueues.get(effectId);
    }

    public static Map<String, GiftQueue> getGiftQueues() {
        return giftQueues;
    }

    public static void registerAll() {
        registerGiftEffect("盲盒", (audience)-> new GiftEffect(audience) {

            private static List<String> effects = new ArrayList<>();

            static {
                effects.add("致盲");
            }

            @Override
            public void apply(Game game) {
                Player player = game.getPlayer();
                String effect = effects.get(RandomUtil.getRandomInt(0, effects.size() - 1));
                addGiftEffect(audience, effect);
            }
            @Override
            public int getQueueTime() {
                return 20;
            }
        });
        registerGiftEffect("致盲", (audience)-> new GiftEffect(audience) {
            @Override
            public void apply(Game game) {
                Player player = game.getPlayer();
                player.addPotionEffect(new PotionEffect(PotionEffectType.BLINDNESS, 20 * 3, 0, false, false));
            }
            @Override
            public int getQueueTime() {
                return 60;
            }
        });
    }

    private static void registerGiftEffect(String id, Function<String, GiftEffect> giftEffectGetter) {
        giftEffectMap.put(id, giftEffectGetter);
    }

}
